Class: Thrift::JSONContext

Inherits:
Object
  • Object
show all
Defined in:
lib/thrift/protocol/json_protocol.rb

Overview

Class to serve as base JSON context and as base class for other context implementations

Direct Known Subclasses

JSONListContext, JSONPairContext

Constant Summary collapse

@@kJSONElemSeparator =
','

Instance Method Summary collapse

Instance Method Details

#escapeNumObject

Return true if numbers need to be escaped as strings in this context. Default behavior is to return false.



70
71
72
# File 'lib/thrift/protocol/json_protocol.rb', line 70

def escapeNum
  return false
end

#read(reader) ⇒ Object

Read context data from the trans. Default is to do nothing.



63
64
# File 'lib/thrift/protocol/json_protocol.rb', line 63

def read(reader)
end

#write(trans) ⇒ Object

Write context data to the trans. Default is to do nothing.



57
58
# File 'lib/thrift/protocol/json_protocol.rb', line 57

def write(trans)
end