Module: Thrift::Struct_Union

Defined in:
lib/thrift/struct_union.rb

Constant Summary collapse

CONTAINER_TYPES =
[]

Instance Method Summary collapse

Instance Method Details

#each_fieldObject



45
46
47
48
49
50
# File 'lib/thrift/struct_union.rb', line 45

def each_field
  sorted_field_ids.each do |fid|
    data = struct_fields[fid]
    yield fid, data
  end
end

#field_info(field) ⇒ Object



156
157
158
159
160
161
162
# File 'lib/thrift/struct_union.rb', line 156

def field_info(field)
  { :type => field[:type],
    :class => field[:class],
    :key => field[:key],
    :value => field[:value],
    :element => field[:element] }
end

#inspect_collection(collection, field_info) ⇒ Object



189
190
191
192
193
194
195
# File 'lib/thrift/struct_union.rb', line 189

def inspect_collection(collection, field_info)
  buf = []
  collection.each do |k|
    buf << inspect_field(k, field_info[:element])
  end
  "[" + buf.join(", ") + "]"
end

#inspect_field(value, field_info) ⇒ Object



164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
# File 'lib/thrift/struct_union.rb', line 164

def inspect_field(value, field_info)
  if enum_class = field_info[:enum_class]
    "#{enum_class.const_get(:VALUE_MAP)[value]} (#{value})"
  elsif value.is_a? Hash
    if field_info[:type] == Types::MAP
      map_buf = []
      value.each do |k, v|
        map_buf << inspect_field(k, field_info[:key]) + ": " + inspect_field(v, field_info[:value])
      end
      "{" + map_buf.join(", ") + "}"
    else
      # old-style set
      inspect_collection(value.keys, field_info)
    end
  elsif value.is_a? Array
    inspect_collection(value, field_info)
  elsif value.is_a? Set
    inspect_collection(value, field_info)
  elsif value.is_a?(String) && field_info[:binary]
    value.unpack("H*").first
  else
    value.inspect
  end
end

#is_container?(type) ⇒ Boolean

Returns:

  • (Boolean)


152
153
154
# File 'lib/thrift/struct_union.rb', line 152

def is_container?(type)
  CONTAINER_TYPES[type]
end

#name_to_id(name) ⇒ Object



24
25
26
27
28
29
30
31
32
33
34
# File 'lib/thrift/struct_union.rb', line 24

def name_to_id(name)
  names_to_ids = self.class.instance_variable_get(:@names_to_ids)
  unless names_to_ids
    names_to_ids = {}
    struct_fields.each do |fid, field_def|
      names_to_ids[field_def[:name]] = fid
    end
    self.class.instance_variable_set(:@names_to_ids, names_to_ids)
  end
  names_to_ids[name]
end

#read_field(iprot, field = {}) ⇒ Object



52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
# File 'lib/thrift/struct_union.rb', line 52

def read_field(iprot, field = {})
  case field[:type]
  when Types::STRUCT
    value = field[:class].new
    value.read(iprot)
  when Types::MAP
    key_type, val_type, size = iprot.read_map_begin
    raise ProtocolException.new(ProtocolException::NEGATIVE_SIZE, 'Negative size') unless size >= 0
    # Skip the map contents if the declared key or value types don't match the expected ones.
    if (size != 0 && (key_type != field[:key][:type] || val_type != field[:value][:type]))
      size.times do
        iprot.skip(key_type)
        iprot.skip(val_type)
      end
      value = nil
    else
      value = {}
      size.times do
        k = read_field(iprot, field_info(field[:key]))
        v = read_field(iprot, field_info(field[:value]))
        value[k] = v
      end
    end
    iprot.read_map_end
  when Types::LIST
    e_type, size = iprot.read_list_begin
    raise ProtocolException.new(ProtocolException::NEGATIVE_SIZE, 'Negative size') unless size >= 0
    # Skip the list contents if the declared element type doesn't match the expected one.
    if (e_type != field[:element][:type])
      size.times do
        iprot.skip(e_type)
      end
      value = nil
    else
      value = []
      size.times do
        value << read_field(iprot, field_info(field[:element]))
      end
    end
    iprot.read_list_end
  when Types::SET
    e_type, size = iprot.read_set_begin
    raise ProtocolException.new(ProtocolException::NEGATIVE_SIZE, 'Negative size') unless size >= 0
    # Skip the set contents if the declared element type doesn't match the expected one.
    if (e_type != field[:element][:type])
      size.times do
        iprot.skip(e_type)
      end
    else
      value = Set.new
      size.times do
        element = read_field(iprot, field_info(field[:element]))
        value << element
      end
    end
    iprot.read_set_end
  else
    value = iprot.read_type(field)
  end
  value
end

#sorted_field_idsObject



36
37
38
39
40
41
42
43
# File 'lib/thrift/struct_union.rb', line 36

def sorted_field_ids
  sorted_field_ids = self.class.instance_variable_get(:@sorted_field_ids)
  unless sorted_field_ids
    sorted_field_ids = struct_fields.keys.sort
    self.class.instance_variable_set(:@sorted_field_ids, sorted_field_ids)
  end
  sorted_field_ids
end

#write_container(oprot, value, field = {}) ⇒ Object



122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
# File 'lib/thrift/struct_union.rb', line 122

def write_container(oprot, value, field = {})
  case field[:type]
  when Types::MAP
    oprot.write_map_begin(field[:key][:type], field[:value][:type], value.size)
    value.each do |k, v|
      write_data(oprot, k, field[:key])
      write_data(oprot, v, field[:value])
    end
    oprot.write_map_end
  when Types::LIST
    oprot.write_list_begin(field[:element][:type], value.size)
    value.each do |elem|
      write_data(oprot, elem, field[:element])
    end
    oprot.write_list_end
  when Types::SET
    oprot.write_set_begin(field[:element][:type], value.size)
    value.each do |v,| # the , is to preserve compatibility with the old Hash-style sets
      write_data(oprot, v, field[:element])
    end
    oprot.write_set_end
  else
    raise "Not a container type: #{field[:type]}"
  end
end

#write_data(oprot, value, field) ⇒ Object



114
115
116
117
118
119
120
# File 'lib/thrift/struct_union.rb', line 114

def write_data(oprot, value, field)
  if is_container? field[:type]
    write_container(oprot, value, field)
  else
    oprot.write_type(field, value)
  end
end